import os
import pysweetter
import twittermod
import ConfigParser

from urllib import urlretrieve

class Backend:
    def __init__(self, config_file):
        self.config_file = config_file
        self.viewed = []
        self.load_conf()

    def load_conf(self):
        self.config = ConfigParser.ConfigParser()
        self.config.read(self.config_file)

        self.cache = self.config.get('General', 'CACHE')
        self.size = int(self.config.get('General', 'SIZE'))
        self.user = self.config.get('General', 'USER')
        self.api = self.config.get('General', 'API')

        self.sweet_check = self.config.get('General', 'sweet')
        self.twitt_check = self.config.get('General', 'twitt')

        # twitter conf

        self.twitt_user = self.config.get('twitter', 'user')
        self.twitt_passwd = self.config.get('twitter', 'passwd')

        self.auth = True
        if self.api == '':
            self.auth = False

        self.sweet = pysweetter.Sweetter(apikey=self.api)
        self.twitt = twittermod.Twitter(self.twitt_user, self.twitt_passwd)

    def save_conf(self):
        self.config.set('General', 'CACHE', self.cache)
        self.config.set('General', 'USER', self.user)
        self.config.set('General', 'API', self.api)
        self.config.set('General', 'SIZE', self.size)

        self.config.set('General', 'sweet', self.sweet_check)
        self.config.set('General', 'twitt', self.twitt_check)

        self.config.set('twitter', 'user', self.twitt_user)
        self.config.set('twitter', 'passwd', self.twitt_passwd)

        self.auth = False

        self.config.write(open(self.config_file, 'w'))

    def post(self, text):
        if not self.auth:
            self.sweet.autenticate(self.api)
            self.twitt.authenticate(self.twitt_user, self.twitt_passwd)
            self.auth = True
        if self.sweet_check == 'true':
            self.sweet.post(text)
        if self.twitt_check == 'true':
            self.twitt.post(text)

    def update(self, other='', show='index', first=False):
        to_ret = []
        get_sweets = self.sweet.get_last_comments
        get_twitts = self.twitt.get_last_followings
        if show == 'index':
            user = 'index'
        elif show == 'home':
            user = self.user
            get_sweets = self.sweet.get_last_followings
        elif show == 'other':
            get_twitts = self.twitt.get_last_comments
            user = other
            if not user: user = 'index'
                
        latest = []
        if self.sweet_check == 'true':
            latest1 = get_sweets(user)
            for i in latest1:
                i.platform = 'sweetter'
                latest.append(i)
        if self.twitt_check == 'true':
            if not self.auth:
                self.twitt.authenticate(self.twitt_user, self.twitt_passwd)
            latest2 = get_twitts(self.twitt_user)
            for i in latest2:
                i.platform = 'twitter'
                latest.append(i)

        def cmp(x, y):
            if x.created > y.created:
                return -1
            elif x.created > y.created:
                return 1
            else:
                return 0


        latest.sort(cmp=cmp, reverse=True)
        for last in latest:
            if not self.viewed or not self.is_viewed(last) or first:
                self.viewed.append(last)

                name = last.user 
                text = last.sweet
                footter = '\n(%s) [from %s]' %\
                        (last.created.ctime(), last.platform)
                platform = last.platform
                image_path = last.avatar

                path = os.path.join(self.cache,
                        os.path.basename(image_path))
                if not os.path.exists(path):
                    urlretrieve(image_path, path)
                
                to_ret.append((path, name, text, footter, platform))

        return to_ret

    def is_viewed(self, sweet):
        n = len(self.viewed)
        for i in xrange(n):
            view = self.viewed[(n-1) - i]
            if view.sweet == sweet.sweet and\
                view.user == sweet.user and\
                view.created == sweet.created:
                return True
        return False

