# -*- coding: utf8 -*-
#!/usr/bin/python
VERSION = '2.0'

import gtk
gtk.gdk.threads_init()

import gtk.glade
import gobject
import pango
import pygtk
try:
    import pynotify
except:
    pynotify = False

import threading
pygtk.require('2.0')

import backend
import time
import os
import sys
import getopt

import trayicon

import ConfigParser

config_file = "sweetgtk.conf"
IMG = 'img'

def insert_one_tag_into_buffer(buffer, name, *params):
    tag = gtk.TextTag(name)
    while(params):
        tag.set_property(params[0], params[1])
        params = params[2:]
    table = buffer.get_tag_table()
    table.add(tag)

class Ventana:
    def __init__(self):
        self.check_new_version()
        SHOW = 'home'
        self.thread_started = False
        self.show = SHOW
        self.first = True

        self.config = ConfigParser.ConfigParser()
        self.config.read(config_file)
        
        API = self.config.get('General', 'API')


        if pynotify:
            pynotify.init("SweetGTK")
        self.gladefile = "sweetgtk.glade"
        self.glade = gtk.glade.XML(self.gladefile)
        self.glade.signal_autoconnect(self)

        self.config_dialog = self.glade.get_widget('config')
        self.window = self.glade.get_widget('window1')
        self.send = self.glade.get_widget('button1')
        self.scroll0 = self.glade.get_widget('scrolledwindow1')
        self.counter = self.glade.get_widget('counter')
        self.other = self.glade.get_widget('other')
        self.sweet_check = self.glade.get_widget('sweetter')
        self.twitt_check = self.glade.get_widget('twitter')

        self.home_button = self.glade.get_widget('home_button')
        self.index_button = self.glade.get_widget('index_button')
        self.other_button = self.glade.get_widget('other_button')
        self.buttons = [self.home_button, self.index_button,
                self.other_button]

        self.progress = self.glade.get_widget('progressbar1')

        self.entry = gtk.TextView()
        self.entry.set_wrap_mode(gtk.WRAP_WORD)
        self.entry.connect('key-press-event', self.key_event_cb)
        self.buffer = self.entry.get_buffer()
        self.mark = self.buffer.create_mark("end", self.buffer.get_end_iter(), False)
        self.scroll0.add(self.entry)
        self.scroll = self.glade.get_widget('scrolledwindow2')
        self.draw_area = gtk.VBox()
        self.scroll.add_with_viewport(self.draw_area)
        self.entry.set_border_window_size(gtk.TEXT_WINDOW_LEFT, 10)

        self.window.show_all()
        self.update_progress = False

        self.backend = backend.Backend(config_file)
        if self.backend.sweet_check == 'true':
            self.sweet_check.set_active(True)
        else:
            self.sweet_check.set_active(False)
        if self.backend.twitt_check == 'true':
            self.twitt_check.set_active(True)
        else:
            self.twitt_check.set_active(False)


        self.update_progress = True
        gobject.timeout_add(100, self.update_progress_cb)
        self.refresh()
        gobject.timeout_add(30000, self.refresh)

    def update_progress_cb(self, data=None):
        if self.update_progress:
            self.progress.show()
            self.progress.pulse()
            return True
        self.progress.hide()
        return False

    def change_view(self, view):
        self.first = True
        self.backend.viewed = []
        if not self.show == view:
            self.show = view
            self.draw_area.destroy()
            self.draw_area = gtk.VBox()
            self.scroll.add_with_viewport(self.draw_area)
            self.refresh()
            self.select_button(view)

            if not self.update_progress:
                self.update_progress = True
                gobject.timeout_add(100, self.update_progress_cb)

    def select_button(self, button):
        for i in self.buttons:
            if i.get_label() != button:
                i.set_relief(gtk.RELIEF_NONE)
            else:
                i.set_relief(gtk.RELIEF_NORMAL)

    def show_other(self, widget, data=None):
        self.change_view('other')

    def show_index(self, widget, data=None):
        self.change_view('index')

    def show_home(self, widget, data=None):
        self.change_view('home')

    def load_img(self, image, stock=False):
        im = gtk.Image()
        if stock:
            im.set_from_stock(image, gtk.ICON_SIZE_SMALL_TOOLBAR)
            return im
        else:
            original = gtk.gdk.pixbuf_new_from_file(image)
            #interp = gtk.gdk.INTERP_NEAREST
            interp = gtk.gdk.INTERP_TILES
            #interp = gtk.gdk.INTERP_BILINEAR
            #interp = gtk.gdk.INTERP_HYPER

            one = original.scale_simple(self.backend.size,
                    self.backend.size, interp)

            im.set_from_pixbuf(one)
            return im

    def create_entry(self, image, name, message, footter='', 
            platform='', error=False):
        sball = os.path.join(IMG, 'sball.png')
        tball = os.path.join(IMG, 'tball.png')

        ball = ''

        hbox = gtk.HBox()

        img = self.load_img(image, error)
        hbox.pack_start(img, expand=False, padding=5)

        texview = gtk.TextView()
        texview.set_editable(False)
        texview.set_wrap_mode(gtk.WRAP_WORD)
        buffer = texview.get_buffer()
        insert_one_tag_into_buffer(buffer, "bold", "weight", pango.WEIGHT_BOLD)  
        insert_one_tag_into_buffer(buffer, "italic", "style", pango.STYLE_ITALIC)
        insert_one_tag_into_buffer(buffer, "x-small", "scale", pango.SCALE_SMALL)

        iter = buffer.get_iter_at_offset(0)
        buffer.insert_with_tags_by_name(iter, name, "bold")
        buffer.insert(iter, ": " + message)
        buffer.insert_with_tags_by_name(iter, footter,
                "x-small", "italic")

        if platform == 'sweetter':
            ball = sball
        elif platform == 'twitter':
            ball = tball

        if ball:
            anchor = buffer.create_child_anchor(iter)
            widget = gtk.Image()
            widget.set_from_file(ball)
            widget.set_padding(2,2)
            texview.add_child_at_anchor(widget, anchor)
            widget.show_all()

        box = gtk.EventBox()
        reply = gtk.Image()
        reply.set_from_stock('gtk-undo', gtk.ICON_SIZE_SMALL_TOOLBAR)
        anchor = buffer.create_child_anchor(iter)
        box.add(reply)
        box.connect('button-press-event', self.reply, name)

        def change_cursor_cb(widget, event, cursor):
            watch = gtk.gdk.Cursor(cursor)
            widget.window.set_cursor(watch)

        box.connect("enter-notify-event", change_cursor_cb, gtk.gdk.LEFT_PTR)
        box.connect("leave-notify-event", change_cursor_cb, gtk.gdk.XTERM)

        texview.add_child_at_anchor(box, anchor)
        box.show_all()

        if not self.first:
            try:
                self.show_bubble(name, message, img)
            except:
                pass

        hbox.add(texview)
        
        sep = gtk.HSeparator()
        self.draw_area.pack_end(hbox, expand=False, padding=5)
        self.draw_area.pack_end(sep, expand=False)
        self.draw_area.show_all()

    def show_bubble(self, title, msg, img):
        if pynotify:
            self.trayicon.set_blinking(True)
            note = pynotify.Notification(title, msg)
            note.set_icon_from_pixbuf(img.get_pixbuf())
            note.attach_to_status_icon(self.trayicon)
            seconds = 10
            note.set_timeout(seconds*1000)
            note.show()


    def key_event_cb(self, widget, data=None):
        text = self.buffer.get_text(self.buffer.get_start_iter(),
                            self.buffer.get_end_iter())
        self.counter.set_text('%d' % (140 - len(text)))

        self.entry.scroll_to_mark(self.mark, 0.05, True, 0.0, 1.0)

    def pressed(self, widget, data=None):
        '''
        Se llama cuando se pulsa el botón envíar.
        Muestra la barra de progreso y añade un timer para actualizarla.
        '''
        text = self.buffer.get_text(self.buffer.get_start_iter(),
                            self.buffer.get_end_iter())

        if not self.update_progress:
            self.update_progress = True
            gobject.timeout_add(100, self.update_progress_cb)
        self.buffer.set_text('')
        self.counter.set_text('%d' % 140)

        if sys.platform == 'win32':
            self.send_threaded(text)
        else:
            t = threading.Thread(target=self.send_threaded, args=(text,))
            t.start()

    def send_threaded(self, text):
        try:
            self.backend.post(text)
        except Exception, e:
            try:
                gtk.gdk.threads_enter()
                self.create_entry('gtk-dialog-error', 'info:', str(e), error=True)
            finally:
                gtk.gdk.threads_leave()

        try:
            gtk.gdk.threads_enter()
            self.refresh()
        finally:
            gtk.gdk.threads_leave()

    def destroy(self, widget, data=None):
        gtk.main_quit()

    def threaded_refresh(self):
        try:
            gtk.gdk.threads_enter()
            other = self.other.get_text()
        finally:
            gtk.gdk.threads_leave()

        try:
            news = self.backend.update(other, self.show, self.first)
        except Exception, e:
            news = []
            try:
                gtk.gdk.threads_enter()
                self.create_entry('gtk-dialog-error', 'info:', str(e), error=True)
            finally:
                gtk.gdk.threads_leave()

        for post in news:
            path, name, text, footter, platform = post
            try:
                gtk.gdk.threads_enter()
                self.create_entry(path, name, text, footter, platform)
            finally:
                gtk.gdk.threads_leave()

        self.first = False
        self.update_progress = False
        self.thread_started = False
        
    def refresh(self):
        '''
        Show sweets from sweetter
        '''
        if not self.thread_started:
            self.thread_started = True
            if sys.platform == 'win32':
                self.threaded_refresh()
            else:
                t = threading.Thread(target=self.threaded_refresh)
                t.start()
        return True

    def focus(self, widget, data=None):
        self.trayicon.set_blinking(False)


    def show_config(self, widget=None, data=None):
        cache = self.glade.get_widget('cache')
        hbox_size = self.glade.get_widget('hbox_size')
        username = self.glade.get_widget('username')
        apikey = self.glade.get_widget('apikey')
        twitt_user = self.glade.get_widget('twitt_user')
        twitt_pass = self.glade.get_widget('twitt_pass')

        size_combo = gtk.combo_box_new_text()
        size_combo.show()
        index = 0
        for i, size in enumerate(['24', '32', '48', '64']):
            size_combo.append_text(size)
            if int(size) == self.backend.size:
                index = i
        size_combo.set_active(index)
        hbox_size.pack_start(size_combo, padding=5)

        cache.set_filename(self.backend.cache)
        username.set_text(self.backend.user)
        apikey.set_text(self.backend.api)
        twitt_user.set_text(self.backend.twitt_user)
        twitt_pass.set_text(self.backend.twitt_passwd)

        result = self.config_dialog.run()
        self.config_dialog.hide()
        size_combo.destroy()

        def get_active_text(combobox):
            model = combobox.get_model()
            active = combobox.get_active()
            if active < 0:
              return None
            return model[active][0]

        if result:
            new_cache = cache.get_filename()
            new_user = username.get_text()
            new_api = apikey.get_text()
            new_size = get_active_text(size_combo)
            new_twitt_user = twitt_user.get_text()
            new_twitt_pass = twitt_pass.get_text()

            self.backend.cache = new_cache
            self.backend.user = new_user
            self.backend.api = new_api
            self.backend.size = int(new_size)
            self.backend.twitt_user = new_twitt_user
            self.backend.twitt_passwd = new_twitt_pass

            self.backend.save_conf()

            show = self.show
            self.show = ''
            self.change_view(show)
        return result

    def reply(self, widget, event, data=None):
        self.buffer.insert(self.buffer.get_end_iter(), "@" + data)

    def check_button_cb(self, widget, data=None):
        if widget.get_name() == 'sweetter':
            self.backend.sweet_check = 'false'
            if widget.get_active():
                self.backend.sweet_check = 'true'
        if widget.get_name() == 'twitter':
            self.backend.twitt_check = 'false'
            if widget.get_active():
                self.backend.twitt_check = 'true'

        self.backend.save_conf()

        if not self.update_progress:
            self.update_progress = True
            gobject.timeout_add(100, self.update_progress_cb)

        self.refresh()

    def check_new_version(self):
        import urllib
        try:
            latest = urllib.urlopen('http://danigm.net/files/sweetgtk-latest').read().strip()
        except:
            return 

        if VERSION < latest:
            dialogo = gtk.MessageDialog(type=gtk.MESSAGE_INFO, 
                    buttons=gtk.BUTTONS_OK,
                    message_format="Hay una nueva versión de sweetgtk - %s" % latest)
            texto = '''Puedes descargarte la nueva versión de:
 http://wiki.sweetter.net/index.php?title=Sweetgtk'''
            dialogo.format_secondary_text(texto)
            dialogo.run()
            dialogo.destroy()

if __name__ == '__main__':
    options, args = getopt.getopt(sys.argv[1:], 'c:')
    for opt in options:
        op, value = opt
        if op == '-c':
            config_file = value

    v = Ventana()
    ti = trayicon.TrayIcon(v)
    v.trayicon = ti
    gtk.main()
